import numpy as np
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
from scipy.stats import gaussian_kde
from astropy.table import Table
from mpl_toolkits.axes_grid1 import make_axes_locatable
from astropy import units as u
from astropy.coordinates import SkyCoord

objectname = input('Please input an object name : ')
filename = 'total_{0}.txt'.format(objectname)
dat = Table.read(filename, format='ascii')

X = dat['RA_1']
Y = dat['Dec_1']

mem_cut = np.where(dat['probs_final'] >= 0.70)
mem_X = dat['RA_1'][mem_cut]
mem_Y = dat['Dec_1'][mem_cut]
cut = np.where(dat['probs_final'] >= 0.0)

XX, YY = np.mgrid[X.min():X.max():100j, Y.min():Y.max():100j]
position = np.vstack([XX.ravel(),YY.ravel()])
values = np.vstack([X,Y])
kernel = gaussian_kde(values)
Z = np.reshape(kernel(position).T, XX.shape)

x_range = (np.max(X) - np.min(X)) / 100
y_range = (np.max(Y) - np.min(Y)) / 100

pdf = kernel(position)
pdf = np.reshape(pdf.T, XX.shape)
pdf /= np.sum(pdf)

num_pdf = pdf*len(X)
num_pdf = num_pdf / (3600*(((np.max(X) - np.min(X))/100) * ((np.max(Y) - np.min(Y))/100)))

fig = plt.figure(figsize=(11,9))
ax = fig.add_subplot(111)

im=ax.imshow(np.rot90(pdf), cmap=plt.cm.gist_earth_r, extent=[np.min(X), np.max(X), np.min(Y), np.max(Y)])

ax.contour(XX, YY, num_pdf, cmap='Blues', levels=20, alpha=0)
cont = ax.contourf(XX, YY, num_pdf, cmap='viridis', levels=30, alpha=0)

ax.tick_params(labelsize=16)
ax.set_xlabel('R.A. (deg)', fontsize=18)
ax.set_ylabel('Dec. (deg)', fontsize=18)

# Find the mean value of KDE
for path in ax.collections[-1].get_paths():
    loc_x, loc_y = path.vertices.mean(axis=0)
    ax.plot(loc_x, loc_y, 'ro' ,ms=6)

sc = ax.scatter(dat['RA_1'][cut], dat['Dec_1'][cut], c=dat['probs_final'][cut] ,cmap='RdPu_r', alpha=0.8, s=4)

if objectname == 'cz30':
    R = 2.3 / 60
elif objectname == 'be34':
    R = 2.5 / 60
elif objectname == 'be75':
    R = 1.9 / 60
elif objectname == 'be76':
    R = 4.0 / 60

circle = plt.Circle((loc_x, loc_y), radius=R, lw=2, fill=False, ec='r', zorder=10)
plt.gca().add_artist(circle)
ax.add_patch(circle)
print('(X,Y) : ',loc_x,loc_y)
c = SkyCoord(ra=loc_x*u.degree, dec=loc_y*u.degree)
print(c.to_string('hmsdms'))
print(c.transform_to('galactic'))

divider = make_axes_locatable(ax)
cax = divider.append_axes('right', size='5%', pad=0.05)
cax2 = divider.append_axes('right',size="5%", pad=0.8)

cb = plt.colorbar(im, cax=cax)
cb2 = plt.colorbar(sc, cax=cax2)
cb.set_label(label='Normalized number density', size=18)
cb.ax.tick_params(labelsize=16)
cb.formatter.set_powerlimits((0, 0))
cb2.set_label(label='Membership probability', size=18)
cb2.ax.tick_params(labelsize=16)
ax.invert_xaxis()
plt.savefig('mem_gaufit_{0}_x{1}_y{2}.pdf'.format(objectname,loc_x,loc_y))
